#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// EhlersSineWave
    /// </summary>
    [Description("EhlersSineWave")]
    public class EhlersSineWave : Indicator
    {
        #region Variables
		private DataSeries phase;
		private DataSeries delta_phase;
		private DataSeries in_phase; 
		private DataSeries quadrature;
		private DataSeries inst_prd;
		private DataSeries back_wtd;
		private DataSeries diff;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Magenta), PlotStyle.Line, "LeadSine"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Sine"));
			
			phase 		= new DataSeries(this);
			in_phase 	= new DataSeries(this); 
			quadrature 	= new DataSeries(this);
			delta_phase = new DataSeries(this);
			inst_prd 	= new DataSeries(this);
			back_wtd 	= new DataSeries(this);
			diff 		= new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBar < 6) 
				return;

			diff.Set(Median[0] - Median[6]);

			double mid = diff[3];
			double wtd = 0.75 * (diff[0] - diff[6]) + 0.25 * (diff[2] - diff[4]);
	
			in_phase.Set(0.33 * mid + 0.67 * in_phase[1]);
			quadrature.Set(0.2 * wtd + 0.8 * quadrature[1]);
			
			if(Math.Abs(in_phase[0] + in_phase[1]) > 0)
				phase.Set(Math.Atan(Math.Abs((quadrature[0] + quadrature[1]) / (in_phase[0] + in_phase[1]))) * 180 / Math.PI);
			
			if(in_phase[0] < 0 && quadrature[0] > 0)
				phase.Set(180 - phase[0]);
			else if(in_phase[0] < 0 && quadrature[0] < 0)
				phase.Set(180 + phase[0]);
			else if(in_phase[0] > 0 && quadrature[0] < 0)
				phase.Set(360 - phase[0]);
			
			delta_phase.Set(phase[1] - phase[0]);
			
			if(phase[1] < 90 && phase[0] > 270)
				delta_phase[0] = 360 + phase[1] - phase[0];
			else if(delta_phase[0] < 1)
				delta_phase.Set(1);			
			else if(delta_phase[0] > 60)
				delta_phase.Set(60);

			double delta_phase_sum = 0;
			inst_prd.Set(0);
			
			for(int i = 0; i <= Math.Min(40, CurrentBar); i++)
			{
				delta_phase_sum = delta_phase_sum + delta_phase[i];
				if(delta_phase_sum > 360 && inst_prd[0] == 0)
					inst_prd.Set(i);
			}
			
			if(inst_prd[0] == 0)
				inst_prd.Set(inst_prd[1]);

			back_wtd.Set(0.25 * inst_prd[0] + 0.75 * back_wtd[1]);
			
			double period 		= Math.Truncate(back_wtd[0]);
			double real_part 	= 0;
			double imag_part 	= 0;
			double dc_phase 	= 0;
			
			for(int i = 0; i <= period -1; i++)
			{
				real_part = real_part + Math.Sin(((360 * i / period) / 180) * Math.PI) * Median[i];
				imag_part = imag_part + Math.Cos(((360 * i / period) / 180) * Math.PI) * Median[i];
			}

			if(Math.Abs(imag_part) > 0.001)
				dc_phase = Math.Atan(real_part / imag_part )* 180 / Math.PI;
			else if(Math.Abs(imag_part) <= 0.001)
				dc_phase = 90 * Math.Sign(real_part);

			dc_phase = dc_phase + 90;

			if(imag_part < 0)
				dc_phase = dc_phase + 180;
			else if(dc_phase > 315)
				dc_phase = dc_phase - 360;

            LeadSine.Set(Math.Sin((dc_phase / 180) * Math.PI));
            Sine.Set(Math.Sin(((dc_phase + 45) / 180) * Math.PI));
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LeadSine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Sine
        {
            get { return Values[1]; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private EhlersSineWave[] cacheEhlersSineWave = null;

        private static EhlersSineWave checkEhlersSineWave = new EhlersSineWave();

        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        public EhlersSineWave EhlersSineWave()
        {
            return EhlersSineWave(Input);
        }

        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        public EhlersSineWave EhlersSineWave(Data.IDataSeries input)
        {

            if (cacheEhlersSineWave != null)
                for (int idx = 0; idx < cacheEhlersSineWave.Length; idx++)
                    if (cacheEhlersSineWave[idx].EqualsInput(input))
                        return cacheEhlersSineWave[idx];

            EhlersSineWave indicator = new EhlersSineWave();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            EhlersSineWave[] tmp = new EhlersSineWave[cacheEhlersSineWave == null ? 1 : cacheEhlersSineWave.Length + 1];
            if (cacheEhlersSineWave != null)
                cacheEhlersSineWave.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheEhlersSineWave = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EhlersSineWave EhlersSineWave()
        {
            return _indicator.EhlersSineWave(Input);
        }

        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        public Indicator.EhlersSineWave EhlersSineWave(Data.IDataSeries input)
        {
            return _indicator.EhlersSineWave(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EhlersSineWave EhlersSineWave()
        {
            return _indicator.EhlersSineWave(Input);
        }

        /// <summary>
        /// EhlersSineWave
        /// </summary>
        /// <returns></returns>
        public Indicator.EhlersSineWave EhlersSineWave(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.EhlersSineWave(input);
        }

    }
}
#endregion
